"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const validate_1 = require("@code-engine/validate");
const ono_1 = require("@jsdevtools/ono");
const worker_1 = require("./worker");
/**
 * Runs CodeEngine plugins on worker threads.
 */
class WorkerPool {
    constructor(engine) {
        /** @internal */
        this._workers = [];
        /** @internal */
        this._isDisposed = false;
        /** @internal */
        this._moduleCounter = 0;
        /** @internal */
        this._roundRobinCounter = 0;
        validate_1.validate.value(engine, "CodeEngine instance");
        validate_1.validate.type.function(engine.emit, "EventEmitter");
        this._cwd = validate_1.validate.string.nonWhitespace(engine.cwd, "cwd");
        let concurrency = validate_1.validate.number.integer.positive(engine.concurrency, "concurrency");
        let emitError = (error) => engine.emit("error" /* Error */, error);
        for (let i = 0; i < concurrency; i++) {
            let worker = new worker_1.Worker(engine.log);
            worker.on("error", emitError);
            this._workers.push(worker);
        }
    }
    /**
     * Indicates the number of worker threads in the pool.
     */
    get size() {
        return this._workers.length;
    }
    /**
     * Indicates whether the `dispose()` method has been called.
     * Once disposed, the `WorkerPool` instance is no longer usable.
     */
    get isDisposed() {
        return this._isDisposed;
    }
    /**
     * Imports the specified `FileProcessor` module in all worker threads.
     *
     * @returns - A proxy function that executes the processor in one of the threads.
     */
    async importFileProcessor(moduleId, data) {
        this._assertNotDisposed();
        let cwd = this._cwd;
        let moduleUID = ++this._moduleCounter;
        let message = { type: "importFileProcessor", cwd, moduleUID, moduleId, data };
        // Import the JavaScript module in all worker threads
        let [name] = await Promise.all(this._workers.map((worker) => worker.importFileProcessor(message)));
        // Create a CodeEngine FileProcessor function that executes the module on a worker thread
        let plugin = {
            [name]: (file, run) => {
                // Select a worker from the pool to process the files
                let worker = this._select();
                // Process the file on the worker thread
                return worker.processFile(moduleUID, file, run);
            }
        };
        // Return the FileProcessor function with the same name as the one in the module
        return plugin[name];
    }
    /**
     * Imports the specified JavaScript module in all worker threads.
     */
    async importModule(moduleId, data) {
        this._assertNotDisposed();
        let cwd = this._cwd;
        let message = { type: "importModule", cwd, moduleId, data };
        // Import the JavaScript module in all worker threads
        await Promise.all(this._workers.map((worker) => worker.importModule(message)));
    }
    /**
     * Terminates all worker threads.
     */
    async dispose() {
        this._isDisposed = true;
        let workers = this._workers;
        this._workers = [];
        await Promise.all(workers.map((worker) => worker.terminate()));
    }
    /**
     * Selects a `Worker` from the pool to perform a task.
     * @internal
     */
    _select() {
        // For now, we just use a simple round-robin strategy,
        // but we may employ a more advanced selection strategy later
        return this._workers[this._roundRobinCounter++ % this._workers.length];
    }
    /**
     * Throws an error if the `WorkerPool` has been disposed.
     * @internal
     */
    _assertNotDisposed() {
        if (this.isDisposed) {
            throw ono_1.ono(`CodeEngine cannot be used after it has been disposed.`);
        }
    }
}
exports.WorkerPool = WorkerPool;
//# sourceMappingURL=worker-pool.js.map