"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("@code-engine/utils");
const ono_1 = require("@jsdevtools/ono");
const path = require("path");
const error_1 = require("../clone/error");
const file_1 = require("../clone/file");
const run_1 = require("../clone/run");
const await_online_1 = require("./await-online");
const messenger_1 = require("./messenger");
const workerScript = path.join(__dirname, "../worker-thread/index.js");
/**
 * Controls an `Executor` instance running on a worker thread.
 * @internal
 */
class Worker extends messenger_1.Messenger {
    constructor(logger) {
        super(workerScript);
        this._logger = logger;
        this._isTerminated = false;
        this._waitUntilOnline = await_online_1.awaitOnline(this);
        this.on("online", this._handleOnline);
        this.on("exit", this._handleExit);
    }
    /**
     * Imports the specified `FileProcessor` module in the worker thread.
     */
    async importFileProcessor(module) {
        await this._waitUntilOnline;
        this._debug(`CodeEngine worker #${this.threadId} is loading ${module.moduleId}`, { moduleId: module.moduleId });
        let reply = await this.postMessageAsync(module);
        return reply.name;
    }
    /**
     * Imports the specified JavaScript module in the worker thread.
     */
    async importModule(module) {
        await this._waitUntilOnline;
        this._debug(`CodeEngine worker #${this.threadId} is importing ${module.moduleId}`, { moduleId: module.moduleId });
        await this.postMessageAsync(module);
    }
    /**
     * Processes the given files in the worker thread.
     */
    async *processFile(moduleUID, file, run) {
        await this._waitUntilOnline;
        this._debug(`CodeEngine worker #${this.threadId} is processing ${file}`, { path: file.path });
        let [fileClone, transferList] = file_1.cloneFile(file);
        let runClone = run_1.cloneRun(run);
        let replies = this.postMessageWithReplies({ type: "processFile", moduleUID, file: fileClone, run: runClone }, transferList);
        for await (let reply of replies) {
            // tslint:disable-next-line: switch-default
            switch (reply.type) {
                case "log":
                    let message = typeof reply.message === "string" ? reply.message : error_1.createError(reply.message);
                    utils_1.log(run.log, reply.level, message, reply.data);
                    break;
                case "file":
                    yield reply.file;
                    break;
            }
        }
    }
    /**
     * Terminates the worker thread and cancels all pending operations.
     */
    async terminate() {
        if (this._isTerminated) {
            return 0;
        }
        this._isTerminated = true;
        let exitCode = await super.terminate();
        return exitCode;
    }
    /**
     * Logs a debug message when the worker thread comes online.
     */
    _handleOnline() {
        this._debug(`CodeEngine worker #${this.threadId} is online`);
    }
    /**
     * Handles the worker thread exiting, either because we told it to terminate, or because it crashed.
     */
    _handleExit(exitCode) {
        let error;
        if (this._isTerminated) {
            // The worker was intentionally terminated
            error = ono_1.ono({ workerId: this.threadId }, "CodeEngine is terminating.");
        }
        else {
            // The worker crashed or exited unexpectedly
            this._isTerminated = true;
            error = ono_1.ono({ workerId: this.threadId }, `CodeEngine worker #${this.threadId} unexpectedly exited with code ${exitCode}.`);
            this.emit("error" /* Error */, error);
        }
        this.rejectAllPendingMessages(error);
        this._debug(`CodeEngine worker #${this.threadId} has terminated`, { exitCode });
    }
    /**
     * Logs a debug message for this worker.
     */
    _debug(message, data) {
        this._logger.debug(message, { ...data, workerId: this.threadId });
    }
}
exports.Worker = Worker;
//# sourceMappingURL=worker.js.map